import java.util.Objects;

/**
 * Encapsulates location, size and speed data of an
 * asteroid object.
 */
public class Asteroid {

    private final double distance;
    private final double theta;
    private final double phi;

    private final int speed;
    private final SizeCategory size;

    public Asteroid(double dist, double t, double p, int speed, SizeCategory size) {
        Objects.requireNonNull(size, "Given size parameter must not be null.");

        distance = dist;
        theta = t;
        phi = p;

        this.speed = speed;
        this.size = size;
    }

    /**
     * Return the Asteroid's distance to earth.
     */
    public double getDistance() {
        return distance;
    }

    /**
     * Return the Asteroid's theta angle in relation to earth's centre.
     */
    public double getTheta() {
        return theta;
    }

    /**
     * Return the Asteroid's phi angle in relation to earth's centre.
     */
    public double getPhi() {
        return phi;
    }

    /**
     * Return the Asteroid's speed towards or away from earth.
     */
    public int getSpeed() {
        return speed;
    }

    /**
     * Return the Asteroid's size category.
     */
    public SizeCategory getSize() {
        return size;
    }

    /**
     * Test equality of object state to the given
     * object instance.
     *
     * @param aThat the instance to compare to
     * @return true if object states are equal
     */
    @Override
    public boolean equals(Object aThat) {
        //check for self-comparison
        if (this == aThat) return true;

        if (!(aThat instanceof Asteroid)) return false;
        //Alternative to the above line :
        //if ( aThat == null || aThat.getClass() != this.getClass() ) return false;

        Asteroid that = (Asteroid) aThat;

        //field-by-field evaluation
        return Double.compare(this.distance, that.distance) == 0 &&
                Double.compare(this.theta, that.theta) == 0 &&
                Double.compare(this.phi, that.phi) == 0 &&
                this.speed == that.speed &&
                this.size.equals(that.size);
    }


    /**
     * Generate a hash code based on the current
     * state of this object.
     *
     * @return this object's hashcode
     */
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 31 * hash + (int) distance;
        hash = 31 * hash + (int) theta;
        hash = 31 * hash + (int) phi;
        hash = 31 * hash + speed;
        hash = 31 * hash + (size == null ? 0 : size.hashCode());
        return hash;

        // alternatively you can use convenient Java API methods
        // return Objects.hash(distance, theta, phi, speed, size);

        // or even have IntelliJ autogenerate hashCode and equals for you
        // see Code -> Generate -> equals and hashcode
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("dist=").append(distance);
        sb.append(", t=").append(theta);
        sb.append(", p=").append(phi);
        sb.append(", speed=").append(speed);
        sb.append(", size=").append(size);
        return sb.toString();
    }
}
